///////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2024, ООО 1С-Софт
// Все права защищены. Эта программа и сопроводительные материалы предоставляются 
// в соответствии с условиями лицензии Attribution 4.0 International (CC BY 4.0)
// Текст лицензии доступен по ссылке:
// https://creativecommons.org/licenses/by/4.0/legalcode
///////////////////////////////////////////////////////////////////////////////////////////////////////

#Область ПрограммныйИнтерфейс

// Позволяет переопределить список доступных интеграций в зависимости от национальной специфики.
// 
// Параметры:
//  ТипыВнешнихСистем - Массив из Строка - "Telegram", "WebChat", "Webhook", "Webhook" и другие.
//
Процедура ПриОпределенииДоступныхИнтеграций(ТипыВнешнихСистем) Экспорт
	
	// Локализация
	ТипыВнешнихСистем.Добавить("VK");
	ТипыВнешнихСистем.Добавить("WhatsApp Devino");
	// Конец Локализация
	
КонецПроцедуры

// Позволяет указать пользовательские инструкции для подключения интеграции с той или иной внешней системой
// в конкретной стране или регионе.
// 
// Параметры:
//  Инструкция - ФорматированнаяСтрока
//  ТипВнешнейСистемы - Строка - "Telegram", "WebChat", "Webhook" или "Webhook".
//
Процедура ПриЗаполненииИнструкцииДляПодключенияИнтеграции(Инструкция, Знач ТипВнешнейСистемы) Экспорт
	
	// Локализация
	Если ТипВнешнейСистемы = ОбсужденияСлужебныйКлиентСервер.ТипыВнешнихСистем().Telegram Тогда
		АдресСсылки = "https://tlgrm.ru/docs/bots#edit-settings";
		ТекстСсылки = НСтр("ru='Инструкция'");
		ДополнитьФорматированнуюСтроку(Инструкция, " " + ТекстСсылки, АдресСсылки);
	ИначеЕсли ТипВнешнейСистемы = ОбсужденияСлужебныйКлиентСервер.ТипыВнешнихСистем().WhatsApp Тогда
		АдресСсылки = "https://v8.1c.ru/static/instruktsiya-po-podklyucheniyu-k-whatsapp-business-api";
		ТекстСсылки = НСтр("ru='Инструкция по подключению к провайдеру'");
		ДополнитьФорматированнуюСтроку(Инструкция, Символы.ПС + ТекстСсылки, АдресСсылки);
	ИначеЕсли ТипВнешнейСистемы = ОбсужденияСлужебныйКлиентСервер.ТипыВнешнихСистем().ВебЧат Тогда
		АдресСсылки = "https://its.1c.ru/db/v83doc#bookmark:dev:TI000002677";
		ТекстСсылки = НСтр("ru='Информация для технического специалиста'");
		ДополнитьФорматированнуюСтроку(Инструкция, Символы.ПС + ТекстСсылки, АдресСсылки);
	ИначеЕсли ТипВнешнейСистемы = ОбсужденияСлужебныйКлиентСервер.ТипыВнешнихСистем().Webhook Тогда
		АдресСсылки = "https://its.1c.ru/db/v8325doc#bookmark:dev:TI000002496";
		ТекстСсылки = НСтр("ru='Информация для технического специалиста'");
		ДополнитьФорматированнуюСтроку(Инструкция, Символы.ПС + ТекстСсылки, АдресСсылки);
	Иначе
		Возврат;
	КонецЕсли;
	// Конец Локализация
	
КонецПроцедуры

#КонецОбласти

// Локализация

#Область СлужебныеПроцедурыИФункции

Процедура ДополнитьФорматированнуюСтроку(ФорматированнаяСтрока, ТекстСсылки, АдресСсылки)
	
	ФорматированныйДокумент = Новый ФорматированныйДокумент;
	ФорматированныйДокумент.УстановитьФорматированнуюСтроку(ФорматированнаяСтрока);

	НовыйЭлемент = ФорматированныйДокумент.Добавить(ТекстСсылки);
	НовыйЭлемент.НавигационнаяСсылка = АдресСсылки;

	ФорматированнаяСтрока = ФорматированныйДокумент.ПолучитьФорматированнуюСтроку();
	
КонецПроцедуры

#КонецОбласти

// Конец Локализация
