///////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2024, ООО 1С-Софт
// Все права защищены. Эта программа и сопроводительные материалы предоставляются 
// в соответствии с условиями лицензии Attribution 4.0 International (CC BY 4.0)
// Текст лицензии доступен по ссылке:
// https://creativecommons.org/licenses/by/4.0/legalcode
///////////////////////////////////////////////////////////////////////////////////////////////////////

#Область ПрограммныйИнтерфейс

// При определении адрес ресурса(URL) картографического сервиса.
// 
// Параметры:
//  ИмяКартографическогоСервиса - Строка - Имя картографического сервиса;
//  АдресРесурса - Строка - адрес ресурса в интернете, ведущий на картографический сайт. В АдресРесурса необходимо обязательно 
//          указать параметр %1, куда будет подставлен искомый адрес для отображения на карте.
//          Например: https://www.openstreetmap.org/search?query=%1.
//
//  Пример:
//     Если ИмяКартографическогоСервиса = "OpenStreetMap" Тогда
//         АдресРесурса = "https://www.openstreetmap.org/search?query=%1";
//     КонецЕсли;
//
Процедура ПриОпределенииURLКартографическогоСервиса(ИмяКартографическогоСервиса, АдресРесурса) Экспорт
	
	// Локализация
		АдресРесурса = "https://maps.yandex.com/?text=%1";
	// Конец Локализация
	
КонецПроцедуры

// Показать адрес на карте Яндекс.
// 
// Параметры:
//  КонтактнаяИнформация - см. УправлениеКонтактнойИнформациейКлиент.ПараметрКонтактнаяИнформацияДляВыполненияКоманд
//  ДополнительныеПараметры - см. УправлениеКонтактнойИнформациейКлиент.ДополнительныеПараметрыДляВыполненияКоманд
//
Процедура ПоказатьАдресНаКартеЯндекс(КонтактнаяИнформация, ДополнительныеПараметры) Экспорт
	
	// Локализация
	УправлениеКонтактнойИнформациейКлиент.ПоказатьАдресНаКарте(КонтактнаяИнформация.Представление, "Яндекс.Карты");
	// Конец Локализация
	
КонецПроцедуры

// Определяет очередность полей в представлении адреса. 
// Вызывается только в международной поставке.
// 
// Параметры:
//  ПорядокПолей - Массив - набор полей в нужном порядке. Например, "area", "ZIPcode", "city".
//  Страна - СправочникСсылка.СтраныМира - ссылка на справочник стран мира
//  ВключатьСтрануВПредставление - Булево - Истина, если в настройках вида контактной информации установлен признак
//                                           включения страны в представление адреса.
//
Процедура ПриОпределенииПорядкаПолейАдреса(ПорядокПолей, Страна, ВключатьСтрануВПредставление) Экспорт
	
	// Локализация
	Если СтрСравнить(Страна, "Перу") = 0 Тогда
		
		ПорядокПолей.Добавить("street");
		ПорядокПолей.Добавить("houseNumber");
		ПорядокПолей.Добавить("city");
		ПорядокПолей.Добавить("area");
		ПорядокПолей.Добавить("zipCode");
		Если ВключатьСтрануВПредставление Тогда
			ПорядокПолей.Добавить("country");
		КонецЕсли;
		
	КонецЕсли;
	// Конец Локализация
	
КонецПроцедуры

#КонецОбласти