///////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2024, ООО 1С-Софт
// Все права защищены. Эта программа и сопроводительные материалы предоставляются 
// в соответствии с условиями лицензии Attribution 4.0 International (CC BY 4.0)
// Текст лицензии доступен по ссылке:
// https://creativecommons.org/licenses/by/4.0/legalcode
///////////////////////////////////////////////////////////////////////////////////////////////////////

#Область ПрограммныйИнтерфейс

// Переопределяет настройки учета оригиналов первичных документов
//
// Параметры:
//  Настройки - Структура:
//   * ВыводитьКнопкиНаФормеДокумента - Булево - Истина, если необходимо отображение внешнего командного интерфейса 
//								по учету оригиналов первичных документов на формах документов в виде кнопок, 
//								Ложь - если в виде гиперссылки. Значение по умолчанию: Ложь.
//   * ОтображатьПодсказкуВФормеИзмененияСостояний - Булево - Ложь, если необходимо убрать информационную надпись
//							    с формы "Изменение состояния оригинала документа" при уточнении состояния по печатным
//								формам. Значение по умолчанию: Истина.
//   * ОткрыватьВыпадающиеМенюПоГиперссылке - Булево - Ложь, если при нажатии на гиперссылку с состоянием оригинала
//								первичного документа в форме списка или документа, необходимо сразу открывать форму
//								"Изменение состояния оригинала документа" для уточнения состояния по печатным формам.
//								Значение по умолчанию: Истина.
//
Процедура ПриОпределенииНастроек(Настройки) Экспорт
	
КонецПроцедуры

// Позволяет доопределить действия при записи общего состояния оригинала.
// 
// Параметры:
//  Документ - ОпределяемыйТип.ОбъектСУчетомОригиналовПервичныхДокументов - ссылка на документ, для которого
//																			записывается новое общее состояние.
//  СостояниеОригинала 	- СправочникСсылка.СостоянияОригиналовПервичныхДокументов - ссылка на устанавливаемое состояние.
//
Процедура ПриИзмененииОбщегоСостоянияОригинала(Документ, СостояниеОригинала) Экспорт
			
КонецПроцедуры

// Позволяет переопределить список объектов печати и печатных форм, перед записью состояний после печати.
// 
//	Параметры:
//  ОбъектыПечати - СписокЗначений - список ссылок на объекты печати.
//  СписокПечати - СписокЗначений - список с именами макетов и представлениями печатных форм.
//
Процедура ПередЗаписьюСостоянийОригиналовПослеПечати(ОбъектыПечати, СписокПечати) Экспорт 
	
КонецПроцедуры

// Определяет объекты конфигурации, в формах списках которых размещены команды учета оригиналов первичных документов,
//
// Параметры:
//  СписокОбъектов - Массив из Строка - менеджеры объектов с процедурой ДобавитьКомандыПечати.
//
Процедура ПриОпределенииОбъектовСКомандамиУчетаОригиналов(СписокОбъектов) Экспорт
	
	

КонецПроцедуры

// Определяет объекты конфигурации, для которых отслеживание состояний будет в разрезе сотрудников.
//
// Параметры:
//  СписокОбъектов - Соответствие из КлючИЗначение:
//          * Ключ - ОбъектМетаданных.
//          * Значение - Строка - наименование табличной части, в которой хранятся сотрудники.
//
Процедура ПриОпределенииМногосотрудниковыхДокументов(СписокОбъектов) Экспорт
	
	

КонецПроцедуры

// Заполняет таблицу значений учета оригиналов
// Если тело процедуры оставить пустым - состояния будут отслеживаться по всем печатным формам подключенных объектов.
// Если в таблицу значений добавить объекты, подключенные к подсистеме учета оригиналов, и их печатные формы,
// то состояния будут отслеживаться только по ним.
//  
// Параметры:
//   ТаблицаУчетаОригиналов - ТаблицаЗначений - коллекция объектов и макетов по которым требуется вести учет оригиналов:
//              * ОбъектМетаданных - ОбъектМетаданных.
//              * Идентификатор - Строка - идентификатор макета.
//
// Пример:
//	 НоваяСтрока = ТаблицаУчетаОригиналов.Добавить();
//	 НоваяСтрока.ОбъектМетаданных = Метаданные.Документы.РеализацияТоваров;
//	 НоваяСтрока.Идентификатор = "РасходнаяНакладная";
//
Процедура ЗаполнитьТаблицуУчетаОригиналов(ТаблицаУчетаОригиналов) Экспорт	
	
	

КонецПроцедуры

#КонецОбласти
