///////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2024, ООО 1С-Софт
// Все права защищены. Эта программа и сопроводительные материалы предоставляются 
// в соответствии с условиями лицензии Attribution 4.0 International (CC BY 4.0)
// Текст лицензии доступен по ссылке:
// https://creativecommons.org/licenses/by/4.0/legalcode
///////////////////////////////////////////////////////////////////////////////////////////////////////

#Если Сервер Или ТолстыйКлиентОбычноеПриложение Или ВнешнееСоединение Тогда
#Область СлужебныеПроцедурыИФункции

// Возвращает признак того, что пользователь поставил отметку "Больше не напоминать" в сообщении о
// необходимости перевыпуска сертификата или снял отметку "Напоминать о готовности" в заявлении на сертификат.
// 
// Параметры:
//  Сертификат - СправочникСсылка.СертификатыКлючейЭлектроннойПодписиИШифрования
// 
// Возвращаемое значение:
//  Булево - если Истина - Оповещен, больше не напоминать
//
Функция ПользовательОповещен(Сертификат) Экспорт

	Запрос = Новый Запрос;
	Запрос.Текст =
	"ВЫБРАТЬ
	|	ОповещенияПользователейСертификатов.Оповещен
	|ИЗ
	|	РегистрСведений.ОповещенияПользователейСертификатов КАК ОповещенияПользователейСертификатов
	|ГДЕ
	|	ОповещенияПользователейСертификатов.Сертификат = &Сертификат
	|	И ОповещенияПользователейСертификатов.Пользователь = &Пользователь";

	Запрос.УстановитьПараметр("Пользователь", Пользователи.ТекущийПользователь());
	Запрос.УстановитьПараметр("Сертификат", Сертификат);
	
	УстановитьПривилегированныйРежим(Истина);
	РезультатЗапроса = Запрос.Выполнить();
	УстановитьПривилегированныйРежим(Ложь);
	ВыборкаДетальныеЗаписи = РезультатЗапроса.Выбрать();
	Если ВыборкаДетальныеЗаписи.Следующий() Тогда
		Возврат ВыборкаДетальныеЗаписи.Оповещен;
	КонецЕсли;

	Возврат Ложь;

КонецФункции

#КонецОбласти
#КонецЕсли